/**
 * @author sergio lisan
 *
 *  The MIT License (MIT)
 *
 *  Copyright (c) 2016 Venosyd Opensource Initiative (Opensyd)
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy of
 *  this software and associated documentation files (the "Software"), to deal in
 *  the Software without restriction, including without limitation the rights to
 *  use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
 *  the Software, and to permit persons to whom the Software is furnished to do so,
 *  subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in all
 *  copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 *  FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 *  COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 *  IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 *  CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 */

library venosyd_shelf_addons;

import 'dart:async';
import 'dart:convert';

import 'package:dson/dson.dart';
import 'package:shelf/shelf.dart';
import 'package:shelf_path/shelf_path.dart' as path;

/// CORS header para operacoes restritas no browser
Map _CORSHeader = {
  'content-type': 'text/json',
  'Access-Control-Allow-Origin': '*',
  'Access-Control-Allow-Headers': "Origin, X-Requested-With, Content-Type, Accept",
  'Access-Control-Allow-Methods': "POST, GET, PUT, DELETE, OPTIONS"
};

/// middleware que implementa o CORS
Middleware corsMiddleWare = createMiddleware(requestHandler: reqCorsHandler, responseHandler: respCorsHandler);

/// request handler do CORS
Response reqCorsHandler(Request request) =>
    request.method == "OPTIONS" ? new Response.ok(null, headers: _CORSHeader) : null;

/// response handler do CORS
Response respCorsHandler(Response response) => response.change(headers: _CORSHeader);

/// funcao que faz a triagem e direcionamento de requests
Future<Response> jsonHandler(Function getter, Request request) {
  return getPostParams(request)
      .then((params) => getPathParams(request, params))
      .then((json) => getter(json))
      .then((list) => dartsonListToJson(list))
      .then(makeResponse);
}

/// funcao que gera uma resposta positiva
Future<Response> makeResponse(String json) async => new Response.ok(json);

/// converter
String dartsonListToJson(payload) => toJson(payload);

/// extrai o payload GET do json do request
Map getPathParams(Request request, Map payload) {
  Map params = path.getPathParameters(request);
  params.forEach((key, val) => payload[key] = val);
  return payload;
}

/// extrai o payload POST do json do request
Future<Map> getPostParams(Request req) => req.readAsString().then((bd) => bd.isNotEmpty ? JSON.decode(bd) : {});